@@ -43,6 +43,7 @@
 #include "event.h"
 #include "adaptors.h"
 #include "object.h"
+#include "common/spi-stateset.h"
 
 #include "accessible-register.h"
 #include "accessible-leasing.h"
@@ -134,6 +135,7 @@ spi_atk_bridge_get_bus (void)
                       (long) BUFSIZ, False,
                       (Atom) 31, &actual_type, &actual_format,
                       &nitems, &leftover, &data);
+  XCloseDisplay (bridge_display);
 
   dbus_error_init (&error);
 
@@ -164,6 +166,7 @@ spi_atk_bridge_get_bus (void)
               g_warning ("AT-SPI: Couldn't register with bus: %s\n", error.message);
               return NULL;
             }
+          dbus_connection_set_exit_on_disconnect (bus, FALSE);
         }
     }
 
